/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2010  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#include "vm.h"

#include <unistd.h>

using namespace std;

#ifdef SIGHANDLE
void handle_sigint(int signum){
  switch(signum){
    case SIGINT:
      if(shared::state==STATE_WORK){
        shared::interrupt=1;
        alarm(2);//wait two seconds, then terminate
      }else
      if(shared::wait_ptr && *shared::wait_ptr){//if waiting for something, cancel wait
        *shared::wait_ptr=0;
#ifdef HAVE_LIBREADLINE
        rl_initialize();
        rl_set_prompt("ised> ");
        printf("\n");
        rl_forced_update_display();
#endif
      }else{//if input line is nonempty, clear it and redisplay prompt
#ifdef HAVE_LIBREADLINE
        if(strlen(rl_line_buffer)>0){
          rl_initialize();
          printf("\n");
          rl_forced_update_display();
        }else{//on empty line, ordinary interrupt
          signal(SIGINT,SIG_DFL);
          raise(SIGINT);
        }
#else
          signal(SIGINT,SIG_DFL); //if no readline, ordinary interrupt
          raise(SIGINT);
#endif
      }
      break;
    case SIGALRM:
      if(shared::state==STATE_WORK){//if after 2 seconds the work didn't stop
        fprintf(stderr,"signal: soft interrupt failed, terminating\n");
        raise(SIGTERM);//terminate
      }
      break;
  }
}
#endif

int main(int argc, char** argz){
  //initializing operators
  fill_ops();

  eval_control evaluator;

  if(argc==1){
    if(isatty(0)){
#ifdef SIGHANDLE
      shared::interactive_flag=1;
      signal(SIGINT,handle_sigint);
      signal(SIGALRM,handle_sigint);
#endif
      evaluator.read_interactive();
      printf("\n");
    }else{
      evaluator.read_file(stdin);
    }
  }else{
    for(int i=1;i<argc;i++){
      evaluator.input_line(argz[i]);
    }
  }

  //killing operators
  kill_ops();

  return 0;
}
