/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2010  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#include "shunting.h"

#include <limits.h>
#include <cstring>

using namespace std;

treenode::treenode(){
  type=NODE_NONE;
}

treenode::treenode(opid o){
  type=NODE_OP;
  op=o;
}

treenode::treenode(value v){
  type=NODE_VAL;
  val=v;
}

op_stub::op_stub(opid o, int i){
  op=o;
  index=i;
}

#if DEBUG>1

void treenode::treeplot(FILE* fajl){
  fprintf(fajl,"<");
  switch(type){
    case NODE_OP:
      fprintf(fajl,"operator type='%s'",op.name);
      break;
    case NODE_VAL:
      fprintf(fajl,"value value='");
      val.print(fajl,"%d","%g");
      fprintf(fajl,"'");
      break;
  }
  fprintf(fajl,">\n");
  for(list<treenode>::iterator i=childz.begin();i!=childz.end();i++){
    i->treeplot(fajl);
  }
  switch(type){
    case NODE_OP: fprintf(fajl,"</operator>\n");break;
    case NODE_VAL: fprintf(fajl,"</value>\n");break;
  }
}

#endif

compiler::compiler(){
}

static int get_offset(uint asoc){
  switch(asoc){
    case ASOC_BLANK:
    case ASOC_LUNARY:
      return 0;
    case ASOC_RUNARY:
    case ASOC_RINFIX:
    case ASOC_LINFIX:
      return 1;
    case ASOC_RBINARY:
      return 2;
    case ASOC_MULTI:
      return INT_MAX;
  }
  return -1;
}

void compiler::stack_putNONE(opid op){
  out_stack.push_back(treenode(op));
}

void compiler::stack_putONE(opid op){
  if(out_stack.size()<1){
    fprintf(stderr,"syntax: operator '%s' has wrong number of arguments, skipping\n",op.name);
    return;
  }
  treenode t=treenode(op);
  t.childz.push_front(out_stack.back());
  out_stack.pop_back();
  out_stack.push_back(t);
}

void compiler::stack_putTWO(opid op){
  if(out_stack.size()<2){
    fprintf(stderr,"syntax: operator '%s' has wrong number of arguments, skipping\n",op.name);
    return;
  }
  treenode t=treenode(op);
  t.childz.push_front(out_stack.back());
  out_stack.pop_back();
  t.childz.push_front(out_stack.back());
  out_stack.pop_back();
  out_stack.push_back(t);
}

void compiler::push_op(opid op){
  switch(op.asoc){
    case ASOC_BLANK://konstante: samo porinemo gor
      //ce je slucajno kdo zafilan
      while(is_overflown()){
        shift_op();
      }
      stack_putNONE(op);
      break;
    case ASOC_RUNARY://ta gre pa na opstack, recimo #a
      while(is_overflown()){//ce bo ta ze cez
        shift_op();
      }
      //dodamo ga
      op_stack.push(op_stub(op,out_stack.size()));
      break;
    case ASOC_LUNARY://recimo a!
      while(top_precedence()>=op.precedence){//dol vrzemo vse, ki so vaznejsi
        shift_op();
      }
      stack_putONE(op);//direkt gor
      break;
    case ASOC_RINFIX://recimo a^b
      while(top_precedence()>op.precedence){//dol vrzemo vse, ki so vaznejsi
        shift_op();
      }
      //dodamo na opstack
      op_stack.push(op_stub(op,out_stack.size()));
      break;
    case ASOC_LINFIX://recimo a+b
      while(top_precedence()>=op.precedence){//dol vrzemo vse, ki so vaznejsi, vkljucno z istimi
        shift_op();
      }
      //dodamo na opstack
      op_stack.push(op_stub(op,out_stack.size()));
      break;
    case ASOC_RBINARY://recimo @ 1 2
      while(is_overflown()){//ce bo cez
        shift_op();
      }
      op_stack.push(op_stub(op,out_stack.size()));
      break;
  }
}

static char* get_par_name(uint type){
  switch(type){
    case OP_LIST:  return shared::shared_ops[0].name;
    case OP_ITER1: return shared::shared_ops[1].name;
    case OP_ITER2: return shared::shared_ops[2].name;
    default: return 0;
  }
}

void compiler::push_par(uint type){//levi oklepaj
  opid op;
  switch(type){
    case OP_LIST:  op=shared::shared_ops[0];break;
    case OP_ITER1: op=shared::shared_ops[1];break;
    case OP_ITER2: op=shared::shared_ops[2];break;
  }
  while(is_overflown()){//ce je slucajno to samo element pri nastevanju
    shift_op();
  }
  op_stack.push(op_stub(op,out_stack.size()));
}

void compiler::pop_par(uint type){//desni oklepaj
  while(top_precedence()){
     shift_op();
  }
  if(op_stack.empty()){//ce ni blo oklepaja gor
    fprintf(stderr,"syntax: operator stack empty\n");
    fprintf(stderr,"syntax: no opening brace of type %s\n",get_par_name(type));
    return;
  }
  op_stub stub=op_stack.top();
  op_stack.pop();
  if(stub.op.code!=type){//ce je {)
    fprintf(stderr,"syntax: mismatched braces of types %s and %s, converting both to %s\n",stub.op.name,get_par_name(type),stub.op.name);
    type=stub.op.code;
  }
  treenode t=treenode(stub.op);
  while(out_stack.size()>stub.index){//vse damo gor
    t.childz.push_front(out_stack.back());
    out_stack.pop_back();
  }
  out_stack.push_back(t);
}

void compiler::push_val(value v){
  while(is_overflown()){//ce je slucajno to samo element pri nastevanju
    shift_op();
  }
  out_stack.push_back(treenode(v));
}

void compiler::shift_op(){//prelozimo na levo stran
  if(op_stack.empty()){
    fprintf(stderr,"syntax: attempting pop from empty stack, ignoring\n");
    return;
  }
  op_stub stub=op_stack.top();
  op_stack.pop();
  if(!stub.op.precedence){//oklepaji ne veljajo
    fprintf(stderr,"syntax: brace treated like value or operator\n");
    fprintf(stderr,"syntax: probably brace mismatch, ignoring %s\n",stub.op.name);
    return;
  }
  //ce slucajno ni prav:
  if(get_offset(stub.op.asoc)!=out_stack.size()-stub.index){
    fprintf(stderr,"syntax: operator '%s' needs %d args, only %d given, skipping\n",stub.op.name,get_offset(stub.op.asoc),out_stack.size()-stub.index);
    return;
  }
  //zdaj pa samo:
  switch(stub.op.asoc){
    case ASOC_BLANK:
      stack_putNONE(stub.op);break;
    case ASOC_RUNARY:
    case ASOC_LUNARY:
      stack_putONE(stub.op);break;
    case ASOC_RINFIX:
    case ASOC_LINFIX:
    case ASOC_RBINARY:
      stack_putTWO(stub.op);break;
  }
}

uchar compiler::top_precedence(){
  if(op_stack.empty())return 0;
  return op_stack.top().op.precedence;
}

int compiler::is_overflown(){
  if(op_stack.empty())return 0;
  op_stub stub=op_stack.top();
  return get_offset(stub.op.asoc)<=out_stack.size()-stub.index;
}

void compiler::process(token t){
  switch(t.type){
    case TOK_OP:
      push_op(t.o);
      break;
    case TOK_PAR_L:
      push_par(t.i);
      break;
    case TOK_PAR_R:
      pop_par(t.i);
      break;
    case TOK_VALUE:
      push_val(t.f);
      break;
  }
}

treenode compiler::compile(vector<token>& tokens){
  for(uint i=0;i<tokens.size();i++){
#ifdef SIGHANDLE
    //interrupt
    if(shared::interrupt){return treenode(shared::shared_ops[0]);}
#endif
    process(tokens[i]);
  }
  while(!op_stack.empty()){
    shift_op();
  }
  treenode t=treenode(shared::shared_ops[0]);
  for(list<treenode>::iterator i=out_stack.begin();i!=out_stack.end();i++){
    t.childz.push_back(*i);
  }
  #if DEBUG>1
  FILE* f=fopen("xmltree.xml","w");
  fprintf(f,"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
  t.treeplot(f);
  fclose(f);
  #endif
  return t;
}
