/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2010  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#ifndef H_SHUNTING
#define H_SHUNTING

#include "base.h"
#include "tokener.h"

#include <stack>
#include <list>

using namespace std;

struct op_stub {
  opid op;

  int index;
  op_stub(opid o, int i);
};

#define NODE_NONE 0
#define NODE_OP   1
#define NODE_VAL  2

struct treenode {
  uchar type;
  opid op;
  value val;
  list<treenode> childz;
  treenode();
  treenode(opid o);
  treenode(value v);
  #if DEBUG>1
  void treeplot(FILE* fajl);
  #endif
};

class compiler {
  private:
    stack<op_stub> op_stack;
    list<treenode> out_stack;

  public:
    compiler();
    void stack_putNONE(opid op);
    void stack_putONE(opid op);
    void stack_putTWO(opid op);


    void push_op(opid op);
    void push_par(uint type);
    void pop_par(uint type);
    void push_val(value v);
    void shift_op();
    uchar top_precedence();
    int is_overflown();
    void process(token t);

    treenode compile(vector<token>& tokens);
};


#endif
