/** ised - generating integer and floating point sequences
    Copyright (C) 2009-2010  Simon Copar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

#ifndef H_VMACHINE
#define H_VMACHINE

#include "base.h"
#include "shunting.h"

#include <vector>
#include <map>

#ifdef HAVE_LIBREADLINE
#  if defined(HAVE_READLINE_READLINE_H)
#    include <readline/readline.h>
#  elif defined(HAVE_READLINE_H)
#    include <readline.h>
#  else
#    undef HAVE_LIBREADLINE
#  endif
#endif
#ifdef HAVE_READLINE_HISTORY
#  if defined(HAVE_READLINE_HISTORY_H)
#    include <readline/history.h>
#  elif defined(HAVE_HISTORY_H)
#    include <history.h>
#  else
#    undef HAVE_READLINE_HISTORY
#  endif
#endif

#define BUF_SIZE 4096

using namespace std;

typedef vector<value> array;

class eval_control {

private:

  map<int,array> memory;
  vector<array> history;

  int toggle_print;

  int mempos;
  char awaiting;
  char* lfile;

  char* delim;
  char* Fd;
  char* Ff;

  int bufsize;
  char* buffer;//buffer for reading files
  char* bufgets(FILE* file);

  array evaluate_value(value v);
  array evaluate_operator(uchar code, vector<array>& args);
  array evaluate(treenode& node);
  array evaluate_code(vector<treenode>& code,int semicoloned);
  void interlace_code(vector<treenode>& code);
  void print_array(array& a, FILE* stream, const char* sep, char end);
  void memory_dump();
  void credits_dump();
  void configure_dump();
  FILE* file_open(char* filename);
  void precompile(char* line, char** comment,vector<treenode>& code, int& semicoloned);

public:

  array evaluate_line(char* line, char** comment);
  void read_interactive();
  void read_file(FILE* f);
  void eval_file(FILE* f);
  void load_file(FILE* f);
  void interlace_line(char* line);
  void interlace_to_file(char* f, FILE* l);
  void apply_line_to_file(char* f, FILE* l);
  void apply_file_to_file(FILE* f, FILE* l);
  void input_line(char* line);
  eval_control();
  ~eval_control();
};



#endif
